//******************************************************************************
//! @file $RCSfile: main_can_bootloader_example.c,v $
//!
//! Copyright (c) 2007 Atmel.
//!
//! Use of this program is subject to Atmel's End User License Agreement.
//! Please read file license.txt for copyright notice.
//!
//! @brief: This application performs a CAN boot loader for AT90CAN128/64/32
//!         with aut-bitrate, IAP (In Application Programming) and API
//!         (Application Programming Interface).
//!
//! This boot loader is an example (application note like), the customer
//! needs to appropriate this code to fit exactly with its application and
//! its environment. Some specific calls (or jump) in customer application
//! are necessary to use correctly the IAP and API features.
//!
//! This file can be parsed by Doxygen for automatic documentation generation.
//! This file has been validated with AVRStudio-413528/WinAVR-20070122.
//!
//! @version $Revision: 3.20 $ $Name: jtellier $
//!
//! @todo
//! @bug
//******************************************************************************

//_____  I N C L U D E S _______________________________________________________
#include "config.h"
#include "can_isp_protocol.h"
#include "isp_lib.h"
#include "reduced_can_lib.h"
#include "can_drv.h"
#include "flash_boot_lib.h"
#include "flash_api_lib.h"

//------------------------------------------------------------------------------
//  @fn main
//!
//! Core of "main_can_boot_loader.c".
//!
//! @brief: This application performs a CAN boot loader for AT90CAN128/64/32
//!         with aut-bitrate, IAP (In Application Programming) and API
//!         (Application Programming Interface).
//!
//! @warning: Don't forget to full locate the boot loader in "Boot Loader
//!           Flash Section".
//!           xxx_HWCB defined in "board.h" file
//!
//! @param none.
//!
//! @return Integer 0
//!
// ----------------------------------------------------------------------------

#define sbi(ADDRESS,BIT) ((ADDRESS) |= (1<<(BIT)))
#define cbi(ADDRESS,BIT) ((ADDRESS) &= ~(1<<(BIT)))

//###############################################################################

int main (void)
{
    U8      u8_temp;
    Bool    auto_b = 1;
//	sbi(DDRG ,2);										// LED - ausgang
//	sbi(PORTG,2);										// LED - OUT

    Disable_interrupt();								// First of all, disabling the Global Interrupt
 
 //################################################################## Start application
   	if ((MCUSR & (1<<PORF)))							// check Status bit Hardware Reset
  		{
        MCUSR = 0; 										// Clear all reset flags
        u8_temp = get_conf_byte(BSB);
 	    if ( (u8_temp!=BSB_DEFAULT) || (u8_temp==BSB_DEFAULT) )
			{
			isp_jump_to ( (((U16)get_conf_byte(SA_H))<<8) | ((U16)get_conf_byte(SA_L)) );   // Start application
        	}
		}

//################################################################## Start Bootloader bei Hardware Reset
	//	cbi(PORTG,2);										// LED - ON
	MCUSR = 0;											// Clear all reset flags
	// Not opened -> Open CAN communication
    can_communication_opened = 0;  					 	
    CAN_PORT_DIR &= ~(1<<CAN_INPUT_PIN );			 	// Pull-up on TxCAN & RxCAN one by one to use bit-addressing
    CAN_PORT_DIR &= ~(1<<CAN_OUTPUT_PIN);
    CAN_PORT_OUT |=  (1<<CAN_INPUT_PIN );
    CAN_PORT_OUT |=  (1<<CAN_OUTPUT_PIN);

    while (1)
    {
        auto_b = ((~auto_b)&0x01);      				// Flip of "auto_b"
		while ((CAN_PORT_IN & (1<<CAN_INPUT_PIN)) != 0);// Wait until activity on RxCAN
	    Can_reset();									// Reset CAN peripheral
        if (get_conf_byte(EB) != 0xFF)					// Set CAN Bit-timming
	        {
            CANBT1 = get_conf_byte(BTC1);       		// Correction: 11 MArch 2007 - JT
            CANBT2 = get_conf_byte(BTC2);      	 		// idem
            CANBT3 = get_conf_byte(BTC3);      			// idem
    	    }
        else 											// Loop on auto-bitrate until it is performed
          	{
            while ((Can_bit_timing(auto_b))==0);        // c.f. macro in "can_drv.h"
        	}
	    // Enable CAN peripheral
        can_clear_all_mob();                        	// c.f. function in "can_drv.c"
        Can_enable();                              		// c.f. macro in "can_drv.h"

		// CAN ISP protocol execution
        // Initialization of CAN ISP PROTOCOL only when CAN communication NOT opened
        if(!can_communication_opened) can_isp_protocol_init();

        // Exit with "0" only if CAN hardware error occurs
        while(can_isp_protocol_task());
    } 
    return(0);
}// End of "main"
